@@ -187,6 +187,8 @@ func loadIndexes() ([]fdroid.App, error) {
 		}
 		for i := range index.Apps {
 			app := index.Apps[i]
+			app.FdroidRepoName = r.ID
+			app.FdroidRepoURL = r.URL
 			orig, e := m[app.PackageName]
 			if !e {
 				m[app.PackageName] = &app
